package com.googlecode.eventio;

import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

import com.googlecode.eventio.config.Config;
import com.googlecode.eventio.config.Configuration;
import com.googlecode.eventio.core.EventMatcher;
import com.googlecode.eventio.core.impl.EventDispatcherImpl;
import com.googlecode.eventio.core.impl.EventExecutorImpl;
import com.googlecode.eventio.core.impl.HashCodeAndEqualsEventMacher;
import com.googlecode.eventio.core.impl.config.ConfigImpl;
import com.googlecode.eventio.core.impl.util.NamedThreadFactory;

/**
 * Copyright 2012 eventio
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @author steph
 * 
 */
public final class Eventio {

	private static Configuration DEFAULT_CONFIGURATION = new Configuration() {

		@Override
		public void configure(Config config) {
			config.threadFactory(new NamedThreadFactory("eventio"));
		}
	};

	private Eventio() {

	}

	public static EventDispatcher createEventEmitter() {
		return createEventEmitter(DEFAULT_CONFIGURATION);
	}

	public static EventDispatcher createEventEmitter(Configuration configuration) {

		ConfigImpl config = new ConfigImpl();

		configuration.configure(config);

		EventMatcher matcher = new HashCodeAndEqualsEventMacher();

		final EventDispatcher dispatcher = new EventDispatcherImpl<Object, Object>(
				config.getThreadFactory(), matcher);

		return dispatcher;
	}

	public static EventExecutor createEventExecuter(EventDispatcher dispatcher) {
		return createEventExecuter(dispatcher, Executors.newSingleThreadExecutor());
	}

	public static EventExecutor createEventExecuter(EventDispatcher dispatcher, Executor executor) {
		return new EventExecutorImpl(dispatcher, executor);
	}

}
