package com.googlecode.eventio.core;

public class Async<T> {

	private final T result;
	private final AsyncException exception;

	public Async(T result) {
		this.result = result;
		this.exception = null;
	}

	public Async(Throwable t) {
		this.exception = new AsyncException(t);
		result = null;
	}

	/**
	 * returns the result, unless some exception arose, in which case the exception is thrown, wrapped in a {@link AsyncException}
	 * 
	 * @return the result, if it could be retrieved
	 * @throws AsyncException
	 *             if an exception was caught while trying to getting this result
	 */
	public T get() throws AsyncException {
		if (exception != null) {
			throw exception;
		}
		return result;
	}

	@Override
	public String toString() {
		if (exception != null) {
			return exception.getCause().toString();
		}
		if (result != null) {
			return result.toString();
		}
		return "null";
	}

	public static <T> Async<T> wrap(T result) {
		return new Async<T>(result);
	}
}
