package com.googlecode.eventio.core;

import java.util.Collection;

import com.googlecode.eventio.Reaction;

/**
 * The {@link EventMatcher} is responsible for storing the reactions registered, and to match events against registered filters.
 * 
 * @author mofleury
 * 
 */
public interface EventMatcher<FILTER, EVENT> {

	/**
	 * Subscribes this reaction to the given event. The reaction will be invoked whenever the event is triggered. Subscriptions can be
	 * canceled using {@link #remove(Object, Reaction)}
	 * 
	 * @param filter
	 *            the event to subscribe to
	 * @param reaction
	 *            the reaction to be executed when the event arises
	 */
	public <T> void register(FILTER filter, Reaction<T> reaction);

	/**
	 * retrieves the reactions that match the given event.
	 * 
	 * @param event
	 * @return the collection of reactions that match the given event
	 */
	public <T> Collection<Reaction<T>> findMatches(EVENT event);

	/**
	 * Removes reaction, if any. Reaction comparison is done by reference. If the reaction is present more than once, only the most recently
	 * added instance will be removed
	 * 
	 * @param filter
	 *            the event to remove reactions from
	 * @param reaction
	 *            the reaction to be removed
	 */
	public <T> void remove(FILTER filter, Reaction<T> reaction);

	/**
	 * removes all reactions that correspond to this filter
	 * 
	 * @param filter
	 */
	public void removeAll(FILTER filter);
}