package com.googlecode.eventio.core;

import com.googlecode.eventio.Reaction;

/**
 * Copyright 2012 eventio
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Asynchronous resource. Accessing this resource will not block the caller, and
 * will execute the associated {@link Reaction} when a value is available. This
 * interface should not be implemented directly ; resource implementors should
 * implement {@link BlockingResource}s instead, and use a
 * {@link ResourceAccessManager} to manage them.
 * 
 * @author mofleury
 * 
 * @param <INPUT>
 *            the resource input type
 * @param <OUTPUT>
 *            the resource output type
 * 
 * @see BlockingResource
 * @see ResourceAccessManager
 */
public interface Resource<INPUT, OUTPUT> {

	/**
	 * requests a value from the resource, with the given input. This call
	 * registers the {@link Reaction} to be executed when the return value is
	 * available.
	 * 
	 * @param input
	 * @param reaction
	 */
	void get(INPUT input, Reaction<Async<OUTPUT>> reaction);

}
