package com.googlecode.eventio.core;


/**
 * the {@link ResourceAccessManager} needs the event dispatcher to execute calls, and thus needs an {@link UniqueMatchGenerator} to create
 * events for these calls.
 * 
 * @author mofleury
 * 
 * @param <FILTER>
 * @param <EVENT>
 */
public interface UniqueMatchGenerator<FILTER, EVENT> {

	/**
	 * 
	 * generates an unique filter/event pair, i.e. the filter will only match the event generated, and the event will only be matched by the
	 * filter generated.
	 * 
	 * @return a holder that contains both the filter and the event.
	 */
	public FilterEventHolder<FILTER, EVENT> generateUniqueMatch();

	public static class FilterEventHolder<FILTER, EVENT> {
		public final FILTER filter;
		public final EVENT event;

		public FilterEventHolder(FILTER filter, EVENT event) {
			super();
			this.filter = filter;
			this.event = event;
		}

	}
}