package com.googlecode.eventio.core.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Deque;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.googlecode.eventio.Reaction;
import com.googlecode.eventio.core.EventMatcher;

public class CompositeEventMatcher implements EventMatcher<CompositeEvent, CompositeEvent> {

	private final Map<String, Map<Object, Set<CompositeEvent>>> index = new HashMap<String, Map<Object, Set<CompositeEvent>>>();

	private final Map<CompositeEvent, Deque<Reaction>> reactions = new HashMap<CompositeEvent, Deque<Reaction>>();

	@SuppressWarnings( { "unchecked", "rawtypes" })
	@Override
	public <T> Collection<Reaction<T>> findMatches(CompositeEvent event) {

		List<Reaction> matchedReactions = new ArrayList<Reaction>();

		Set<CompositeEvent> partialMatches = new HashSet<CompositeEvent>();

		// gather all filters for which at least one condition is satisfied
		for (Map.Entry<String, Object> e : event.getSubEvents().entrySet()) {
			Map<Object, Set<CompositeEvent>> withKey = index.get(e.getKey());
			if (withKey != null) {
				Set<CompositeEvent> withValue = withKey.get(e.getValue());
				if (withValue != null) {
					partialMatches.addAll(withValue);
				}
			}
		}

		for (CompositeEvent filter : partialMatches) {
			// check if remaining conditions are also satisfied
			boolean allMatch = true;
			for (Map.Entry<String, Object> subEvent : filter.getSubEvents().entrySet()) {
				Object testValue = event.getSubEvents().get(subEvent.getKey());
				if ((testValue == null) || !testValue.equals(subEvent.getValue())) {
					allMatch = false;
					break;
				}
			}
			if (allMatch) {
				matchedReactions.addAll(reactions.get(filter));
			}
		}

		return (List) matchedReactions;
	}

	@Override
	public <T> void register(CompositeEvent filter, com.googlecode.eventio.Reaction<T> reaction) {

		Deque<Reaction> r = reactions.get(filter);
		if (r == null) {
			r = new LinkedList<Reaction>();
			reactions.put(filter, r);
		}
		r.add(reaction);

		// add filter to index : add it to the set of filters for each key and subEvent
		for (Map.Entry<String, Object> subEvent : filter.getSubEvents().entrySet()) {
			Map<Object, Set<CompositeEvent>> m = index.get(subEvent.getKey());
			if (m == null) {
				m = new HashMap<Object, Set<CompositeEvent>>();
				index.put(subEvent.getKey(), m);
			}
			Set<CompositeEvent> holders = m.get(subEvent.getValue());
			if (holders == null) {
				holders = new HashSet<CompositeEvent>();
				m.put(subEvent.getValue(), holders);
			}
			holders.add(filter);
		}

	};

	@Override
	public <T> void remove(CompositeEvent filter, Reaction<T> reaction) {

		Deque<Reaction> r = reactions.get(filter);
		if (r != null) {
			Iterator<Reaction> iter = r.descendingIterator();
			while (iter.hasNext()) {
				if (iter.next() == reaction) {
					iter.remove();
					break;
				}
			}
			if (r.isEmpty()) {
				reactions.remove(filter);
			}
		}

		for (Map.Entry<String, Object> subEvent : filter.getSubEvents().entrySet()) {
			Map<Object, Set<CompositeEvent>> m = index.get(subEvent.getKey());
			if (m != null) {
				Set<CompositeEvent> filters = m.get(subEvent.getValue());
				filters.remove(filter);
				if (filters.isEmpty()) {
					m.remove(subEvent.getValue());
				}
				if (m.isEmpty()) {
					index.remove(subEvent.getKey());
				}
			}
		}
	}

	@Override
	public void removeAll(CompositeEvent filter) {
		reactions.remove(filter);
	}
}
