/**
 * 
 */
package com.googlecode.eventio.core.impl;

import java.util.concurrent.Callable;
import java.util.concurrent.Executor;

import com.googlecode.eventio.CallableReaction;
import com.googlecode.eventio.EventDispatcher;
import com.googlecode.eventio.EventExecutor;
import com.googlecode.eventio.Reaction;
import com.googlecode.eventio.Result;

/**
 * @author steph
 * 
 */
public class EventExecutorImpl implements EventExecutor {

	private final EventDispatcher dispatcher;
	private final Executor executor;

	public EventExecutorImpl(EventDispatcher dispatcher, Executor executor) {
		this.dispatcher = dispatcher;
		this.executor = executor;
	}

	/**
	 * 
	 * @see java.util.concurrent.Executor#execute(java.lang.Runnable)
	 */
	@Override
	public void execute(Runnable command) {
		executor.execute(command);
	}

	/**
	 * 
	 * @see com.googlecode.eventio.EventExecutor#execute(java.util.concurrent.Callable
	 *      , com.googlecode.eventio.Reaction)
	 */
	@Override
	public <V> void execute(final Callable<V> c, final Reaction<Result<V>> react) {
		execute(new CallableReaction<V>() {

			@Override
			public V call() throws Exception {
				return c.call();
			}

			@Override
			public void react(Result<V> input) {
				react.react(input);
			}
		});
	}

	/**
	 * 
	 * @see com.googlecode.eventio.EventExecutor#execute(com.googlecode.eventio.
	 *      CallableReaction)
	 */
	@Override
	public <V> void execute(final CallableReaction<V> callReac) {

		executor.execute(new Runnable() {

			@Override
			public void run() {
				Result<V> result;

				try {
					result = new Result<V>(callReac.call());
				} catch (Exception e) {
					result = new Result<V>(e);
				}

				Object event = new Object();
				dispatcher.once(event, callReac);
				dispatcher.emit(event, result);
			}
		});

	}

}
