package com.googlecode.eventio.core.impl;

import java.util.Deque;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import com.googlecode.eventio.Reaction;
import com.googlecode.eventio.core.EventMatcher;
import com.googlecode.eventio.core.impl.util.EmptyDeque;


/**
 * {@link EventMatcher} that matches events against filters using hashCode and equals, i.e. using the same contract as {@link HashMap}, and
 * using filters and events as keys in the map.
 * 
 * @author mofleury
 * 
 */
public class HashCodeAndEqualsEventMacher implements EventMatcher<Object, Object> {

	@SuppressWarnings("rawtypes")
	private final Map<Object, Deque<Reaction>> registeredReactions = new HashMap<Object, Deque<Reaction>>();

	@SuppressWarnings("rawtypes")
	@Override
	public <T> void register(Object event, Reaction<T> reaction) {
		Deque<Reaction> registered = registeredReactions.get(event);
		if (registered == null) {
			registered = new LinkedList<Reaction>();
			registeredReactions.put(event, registered);
		}
		registered.add(reaction);

	}

	@Override
	public <T> Deque<Reaction<T>> findMatches(Object event) {
		@SuppressWarnings({ "unchecked", "rawtypes" })
		Deque<Reaction<T>> registered = (Deque) registeredReactions.get(event);
		if (registered == null) {
			return EmptyDeque.emptyDeque();
		}
		return registered;
	}

	@Override
	public <T> void remove(Object event, Reaction<T> reaction) {
		Iterator<Reaction<T>> iter = this.<T> findMatches(event).descendingIterator();
		while (iter.hasNext()) {
			if (iter.next() == reaction) {
				iter.remove();
				break;
			}
		}

		if (registeredReactions.get(event).isEmpty()) {
			registeredReactions.remove(event);
		}
	}

	@Override
	public void removeAll(Object event) {
		registeredReactions.remove(event);
	}

}
